/*
	File:		FWList.h

	Contains:	Primitive linked list class

	Written by:	Richard Rodseth

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <1>	10/24/94	jpa		first checked in
		 <5>	 9/30/94	RR		#1167950. Added RemoveCurrent, fNext and
									fPrevious to FW_CPrivLinkedListIterator
		 <4>	  6/9/94	RR		Remove ASLM stuff
		 <2>	 5/10/94	RR		Removed ASLM_COMPATIBLE and CDECL
		 <1>	  5/5/94	CG		first checked in
		<10>	 3/15/94	MB		Changes to support SCpp/ASLM builds,
									#1150864.
		 <9>	  2/7/94	JA		Tiger Team Makeover. Added some inline
									helpers to eliminate cfront warnings in
									const methods.
		 <8>	 1/31/94	JA		Added more methods to FW_CPrivLink and FW_CPrivLinkedList.
									Inlined some getters/setters.
		 <7>	10/14/93	RR		Added fSeed to detect when iterator is out of sync
		 <6>	 10/6/93	PH		Make destructor virtual
		 <5>	 8/13/93	RR		Added DeleteLinks method
		 <5>	 8/13/93	RR		Added DeleteLinks method
		 <4>	  8/9/93	NP		Added class id string.
		 <3>	  4/27/93	RCR		Rename Includes
		 <2>	  4/27/93	RCR		Include ODTypes and use ODBoolean and kODNULL
		 <1>	  4/27/93	RCR		first checked in
	To Do:
*/

#ifndef FWLIST_H
#define FWLIST_H

#ifndef FWSTDDEF_H
#include "FWStdDef.h"
#endif

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export on
#endif

//=====================================================================================
// Theory of Operation
//=====================================================================================

/*

 Note: These classes are private to the implementation. They are not available
 to part handlers.

 Note: These are primitive classes for implementing higher-level collections
 For example, to create a FrameList class, subclass FW_CPrivLink to add a field to
 store the frame. The FrameList class would use a FW_CPrivLinkedList in its implementation
 and would manufacture the FW_CPrivLink objects internally.
 
 Note: SimpleLinkedList is a list without a "seed" for consistency checking.
 It cannot be iterated over.

*/

//==============================================================================
// Constants
//==============================================================================

//==============================================================================
// Scalar Types
//==============================================================================

//==============================================================================
// Classes defined in this interface
//==============================================================================

class FW_CPrivLink;
class FW_CPrivLinkedList;
class FW_CPrivLinkedListIterator;

//==============================================================================
// Classes used by this interface
//==============================================================================


//==============================================================================
// FW_CPrivLink
//==============================================================================

class FW_CLASS_ATTR FW_CPrivLink {
	public:
	
		FW_CPrivLink();
		
		FW_CPrivLink(FW_CPrivLink* next, FW_CPrivLink* previous);
		
		FW_CPrivLink( const FW_CPrivLink& );
		
		virtual ~FW_CPrivLink();
		
		FW_CPrivLink* GetNext() const					{return fNext;}
		
		FW_CPrivLink* GetPrevious() const				{return fPrevious;}
	
	// The following operations are provided for efficiency, but DO NOT USE THEM
	// if there are any iterators active on a list. These operations don't bump
	// the list's fSeed and the iterators will not be able to detect that they
	// are out of sync!
		
		void  Remove( );
		
		void  AddBefore( FW_CPrivLink *aLink );
		
		void  AddAfter( FW_CPrivLink *aLink );
	
  //private-by-convention:
  		
		void  SetNext(FW_CPrivLink* aLink)				{fNext = aLink;}
		
		void  SetPrevious(FW_CPrivLink* aLink)			{fPrevious = aLink;}

	private:
	
		FW_CPrivLink*		fNext;
		FW_CPrivLink*		fPrevious;
};


//==============================================================================
// FW_CPrivLinkedList
//==============================================================================


class FW_CLASS_ATTR FW_CPrivLinkedList {

	public:
	
	  	FW_CPrivLinkedList();
	  	
	  	~FW_CPrivLinkedList();
	  	
	  	FW_Boolean		IsEmpty( )										const;
	  	
	  	unsigned long	Count()											const;
	  	
	  	FW_Boolean		Includes( const FW_CPrivLink* )							const;
	  	
	  	void			Remove(FW_CPrivLink&);
	  	
	  	void			RemoveAll();

	  	void			DeleteAllLinks();
	  	
	  	FW_CPrivLink*			RemoveFirst();
	  	
	  	FW_CPrivLink*			RemoveLast();
	  	
	  	void			AddBefore(FW_CPrivLink& existing, FW_CPrivLink* link);
	  	
	  	void			AddAfter(FW_CPrivLink& existing, FW_CPrivLink* link);
	  	
	  	void			AddFirst(FW_CPrivLink* link);
	  	
	  	void			AddLast(FW_CPrivLink* link);
	  	
	  	void			AddLast( FW_CPrivLinkedList &list );
	  	
	  	void			AddLastUnique( FW_CPrivLinkedList &list );
	  	
	  	FW_CPrivLink*			After(const FW_CPrivLink& link)							const;
	  	
	  	FW_CPrivLink* 			Before(const FW_CPrivLink& link)						const;
	  	
	  	FW_CPrivLink*			First()											const;
	  	
	  	FW_CPrivLink* 			Last()											const;

protected:

		FW_CPrivLink			fSentinel;	// Marks the head & tail
		unsigned long	fSeed;		// Used to detect out-of-sync iterators
		
		FW_CPrivLink*			GetSentinel( )
												{return &fSentinel;}
		const FW_CPrivLink*		GetSentinel( )									const
												{return &fSentinel;}
		FW_Boolean		IsSentinel( const FW_CPrivLink* link )					const
												{return link==this->GetSentinel();}
		FW_Boolean		NotSentinel( const FW_CPrivLink* link )					const
												{return link!=this->GetSentinel();}

private:			  	
		friend class FW_CPrivLinkedListIterator;
};


//=====================================================================================
// FW_CPrivLinkedListIterator
//=====================================================================================

class FW_CLASS_ATTR FW_CPrivLinkedListIterator {

	public:
	
		FW_CPrivLinkedListIterator(FW_CPrivLinkedList*	list);
		
		~FW_CPrivLinkedListIterator();
		
		FW_CPrivLink*			First();
		
		FW_CPrivLink*			Next();
		
		FW_CPrivLink*			Last();
		
		FW_CPrivLink*			Previous();

		FW_CPrivLink*			Current();

		FW_Boolean 		IsNotComplete();
		
		void			RemoveCurrent();
		
	private:
	
		FW_CPrivLinkedList*		fList;
		FW_CPrivLink*			fCurrent;
		FW_CPrivLink*			fNext;		// Used only when deleting while iterating
		FW_CPrivLink* 			fPrevious;	// Used only when deleting while iterating
		FW_CPrivLink*			fSentinel;
		unsigned long	fSeed;		// Used to detect out-of-sync iterators
		
};

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export off
#endif

#endif // FWLIST_H
